package {

	import net.flashpunk.*
	import net.flashpunk.masks.*
	import net.flashpunk.utils.*
	import net.flashpunk.graphics.*

	// Combined dimension

	public class Combined extends Dimension {

		public var dimensions:Array = [];

		public function Combined() {
			super(null, null)
		}

		public override function get width():int { return map?map.width/24:0 };
		public override function get height():int { return map?map.height/24:0 };

		/** Does tile t1 override tile t0 when blending */
		public function better(t0:int, t1:int): Boolean {
			if(t1<0) return false;
			else return t0<0 || Game.tileWeights[t0] <= Game.tileWeights[t1];
		}

		/** Add a dimension to the combination */
		public function addDimension(d:Dimension): Boolean {
			if(map==null) { setDimension(d); return true; }
			if(dimensions.indexOf(d)>=0) return false; // Already here!

			// Resize maps
			var left:int   = Math.min(x, d.x);
			var top:int    = Math.min(y, d.y);
			var right:int  = Math.max(x+width, d.x+d.width);
			var bottom:int = Math.max(y+height, d.y+d.height);
			if(left<x || top<y || right>x+width || bottom>y+height) {
				resizeBuffers(left, right, top, bottom);
			}
			// Blend in new tiles
			for(var cx:int=d.x; cx<=d.x+d.width; ++cx) {
				for(var cy:int=d.y; cy<=d.y+d.height; ++cy) {
					var t0:int = getTile(cx, cy);
					var t1:int = d.getTile(cx, cy);
					if( better(t0,t1) ) {
						trace(cx,cy," : ", t0, t1, better(t0,t1));
						map.setTile(cx-x, cy-y, t1);
						clip.setTile(cx-x, cy-y,  d.clip.getTile(cx-d.x, cy-d.y) );
					}
				}
			}
			dimensions.push(d);
			return true;
		}

		/** Remove a dimension */
		public function removeDimension(d:Dimension): Boolean {
			if(dimensions.indexOf(d)<0) return false; // Not in
			if(dimensions.length==1) { clearDimensions(); return true; }
			FP.remove(dimensions, d);
			//Re-blend from all other dimensions
			for(var cx:int=d.x; cx<=d.x+d.width; ++cx) {
				for(var cy:int=d.y; cy<=d.y+d.height; ++cy) {
					var t:int = -1;
					for each(var g:Dimension in dimensions) {
						var t1:int = g.getTile(cx,cy);
						if(better(t, t1)) {
							t = t1;
							map.setTile(cx-x, cy-y, t1);
							clip.setTile(cx-x, cy-y,  g.clip.getTile(cx-g.x, cy-g.y) );
						}
					}
					// Void
					if(t<0) {
						map.setTile(cx-x, cy-y, 0);
						map.clearTile(cx-x, cy-y);
						clip.setTile(cx-x, cy-y, true);
					}
				}
			}
			return true;
		}

		/** Add dimension if ampty */
		public function setDimension(d:Dimension): void {
			trace("setDimension", d.name, d.x, d.y, d.width, d.height);

			map = d.map.getSubMap(0,0,d.width, d.height);
			for(var cx:int=0; cx<map.width/24; ++cx) for(var cy:int=0; cy<map.height/24; ++cy) {
				map.setTile(cx,cy, map.getTile(cx,cy));
			}
			map.x = d.x*24; map.y = d.y*24;

			clip = new Grid(d.width*24, d.height*24, 24,24, d.x*24, d.y*24);
			clip.data.copyPixels(d.clip.data, Game.sRect(0,0,d.width,d.height), FP.zero);

			x = d.x; y = d.y;
			dimensions.length = 0;
			dimensions.push(d);
		}
		
		/** Remove all dimensions */
		public function clearDimensions(): void {
			dimensions.length = 0;
			map = null;
			clip = null;
			x = 0; y = 0;
		}

		/** Resize buffers */
		public function resizeBuffers(left:int, right:int, top:int, bottom:int): void {
			//Resize clipmap
			var nc:Grid = new Grid((right-left)*24, (bottom-top)*24, 24,24, left*24, top*24);
			nc.data.copyPixels(clip.data, Game.sRect(0,0,width,height), Game.sPoint(x-left, y-top));
			clip = nc;

			// Resize Tilemap
			map = map.getSubMap(left-x, top-y, right-left, bottom-top);
			map.x = left * 24; map.y = top * 24;

			x = left; y = top;
		}


	}



}
