package {

	import net.flashpunk.*
	import net.flashpunk.utils.*
	import net.flashpunk.masks.*
	import net.flashpunk.graphics.*
	import flash.display.BitmapData;

	public class Dimension {

		public var name:String;
		public var x: int;
		public var y: int;
		public var colour:uint;

		public var source:Tiled;
		public var map:Tilemap;
		public var clip:Grid;
		public var tileset:BitmapData;

		public var objects:Array;

		public function Dimension(data:Tiled, tileset:Class) {
			if(this is Combined) return;
			source = data;
			this.tileset = FP.getBitmap(tileset);
			name = data.property("name");
			x = int(data.property("offsetX"));
			y = int(data.property("offsetY"));
			colour = uint(data.property("colour"));
			if(!colour) colour = FP.getColorHSV(FP.random, 0.5, 1);

			map = data.createTilemap("Map", tileset, "tiles");
			clip = data.createClipmap("Clip", "clip", 0);
			if(!map) trace("Error: Map", name, "is null");
			if(!clip) trace("Error: Clip", name, "is null");
			map.x = clip.x = x * 24;
			map.y = clip.y = y * 24;

			// Tint map
			map.color = colour;
			map.drawGraphic(-map.x, -map.y, map);
			map.color = 0xffffff;

		}

		public function get width(): int { return source.width; }
		public function get height(): int { return source.height; }

		/** Add an object to this dimension */
		public function addObject(o:GameObject): void {
			objects.push(o);
			o.dimension = this;
		}
		/** Remove an object */
		public function removeObject(o:GameObject): void {
			FP.remove(objects, o);
			o.dimension = null;
		}

		/** Create all the objects in this dimension */
		public function createObjects(): void {
			objects = [];
			var list:Array = source.getObjects();
			for each(var o:Object in list) {
				switch(String(o.type)) {
				case "Orb": addObject( new Orb(o.x+map.x, o.y+map.x, o.name) );
				case "Player": break;
				case "End": addObject( new EndPoint(o.x+map.x, o.y+map.y) );
				default: trace("Undefine object type", o.type);
				}
			}
		}

		public function getTile(tx:int, ty:int): int {
			if(tx<x || ty<y || tx>=x+width || ty>=y+height) return -1;
			else return map.getTile(tx-x, ty-y);
		}



		

		

		


	}

}
