package {
	
	import net.flashpunk.*
	import net.flashpunk.utils.*
	import net.flashpunk.graphics.*

	import flash.geom.Rectangle;
	import flash.geom.Point;


	public class Game extends World {

		[Embed(source="data/map0.tmx", mimeType="application/octet-stream")] private static var map0_data: Class;
		[Embed(source="data/map1.tmx", mimeType="application/octet-stream")] private static var map1_data: Class;
		[Embed(source="data/map2.tmx", mimeType="application/octet-stream")] private static var map2_data: Class;
		[Embed(source="data/tiles.png")] public static var tileset:Class;

		public static var tileWeights: Array = [ 0,20,19, 5,5,5, 5,10,5, 5,5,5 ];
		
		public var dimensions:Object;

		public var combined:Combined;
		public var player:Player;
		public var map:Entity;

		public function Game() {
			//Load the maps
			dimensions = {};
			var d:Dimension;
			d = loadMap(map0_data); dimensions[d.name] = d;
			d = loadMap(map1_data); dimensions[d.name] = d;
			d = loadMap(map2_data); dimensions[d.name] = d;

			combined = new Combined();
			map = new Entity();
			map.type = "solid";
			map.layer = 1;
			add(map);

			addDimension( dimensions["start"] );


			// Create a player
			player = new Player(100,100);
			add(player);
			
		}

		public override function begin(): void {
			// Give the player the starting orb
			add( new SelOrb( new Orb(0,0,"start") ));
		}

		public function loadMap(data:Class): Dimension {
			var src:Tiled = new Tiled( FP.getXML(data) );
			var map:Dimension = new Dimension(src, tileset);
			return map;
		}

		public function addDimension(d:Dimension): void {
			if(combined.addDimension(d)) {
				map.mask = combined.clip;
				map.graphic = combined.map;
				if(!d.objects) d.createObjects();
				addList(d.objects);

				//Remove any windows to active dimensions
				var list:Array = [];
				getClass(Orb, list);
				for each(var o:Orb in list) {
					if(o.window.world && combined.dimensions.indexOf(o.target)>=0) {
						remove(o.window);
					}
				}
			}
		}

		public function removeDimension(d:Dimension): void {
			if(combined.removeDimension(d)) {
				removeList(d.objects);
				map.mask = combined.clip;
				map.graphic = combined.map;
			}
		}

		public override function update(): void {
			super.update();
			
			camera.x = FP.lerp(camera.x, player.x-FP.halfWidth, 1);
			camera.y = FP.lerp(camera.y, player.y-FP.halfWidth, 1);


			if(Input.pressed(Key.SPACE)) {
				addDimension( dimensions[ "map1" ] );
			}

		}

		public override function render(): void {
			Draw.setTarget(FP.buffer);
			Draw.rect(0,0,FP.width,FP.height,0);
			Draw.resetTarget();

			super.render();
		}



		/** Utility functions */
		public static var rect:Rectangle = new Rectangle();
		public static var point:Point = new Point();
		public static function sPoint(x:int, y:int): Point {
			point.x = x;
			point.y = y;
			return point;
		}
		public static function sRect(x:int, y:int, w:int, h:int): Rectangle {
			rect.x = x;
			rect.y = y;
			rect.width = w;
			rect.height = h;
			return rect;
		}

	}

}

