package {

	import net.flashpunk.*
	import net.flashpunk.utils.*
	import net.flashpunk.graphics.*

	import flash.display.BitmapData;

	public class Orb extends GameObject {

		[Embed(source="data/orb.png")] public static var orbImage: Class;

		public static var alphaBlock:BitmapData;

		public var targetName:String;
		public var radius: Number = 100;
		public var window: Entity;

		public function Orb(x:Number, y:Number, target:String) {
			super(x,y);
			targetName = target;
			graphic = new Image(orbImage);
			setHitbox(8,8,-6,-6);
			type = "orb";
		}

		public override function added(): void {
			(graphic as Image).color = target.colour;

			window = createWindow();
			world.add(window);
		}
		public override function removed(): void {
			if(window && window.world) window.world.remove(window);
		}

		/** Get the target dimension */
		public function get target(): Dimension {
			if(world && world is Game) return (world as Game).dimensions[targetName];
			else if(FP.world is Game) return (FP.world as Game).dimensions[targetName];
			else return null;
		}

		/** Get alpha value at a position */
		public function getValue(px:Number, py:Number): Number {
			if(px<x-radius || px>x+radius || py<y-radius || py>y+radius) return 0;
			var d:Number = FP.distance(x,y,px,py) / radius;
			if(d<1) return 1-d*d*d*d*d;
			else return 0;
		}

		/** Create the window image */
		public function createWindow(): Entity {
			if(!alphaBlock) alphaBlock = new BitmapData(24,24,true,0x0);
			var x0:int = Math.floor((x-radius)/24);
			var y0:int = Math.floor((y-radius)/24);
			var x1:int = Math.ceil((x+radius)/24);
			var y1:int = Math.ceil((y+radius)/24);
			var bmp:BitmapData = new BitmapData((x1-x0)*24, (y1-y0)*24, true, 0x0);
			// Copy tiles from dimension
			Draw.setTarget(bmp, Game.sPoint(x0*24, y0*24));
			trace(Game.point);
			Draw.graphic(target.map);
			Draw.resetTarget();

			// Now play with some alpha
			FP.rect.x=FP.rect.y=0; FP.rect.width=FP.rect.height=24;
			for(var ix:int=x0; ix<=x1; ++ix) {
				for(var iy:int=y0; iy<=y1; ++iy) {
					var px:int = (ix-x0)*24;
					var py:int = (iy-y0)*24;
					alphaBlock.fillRect( FP.rect, uint( getValue(ix*24+12, iy*24+12)*128)<<24 );
					bmp.copyPixels(bmp, Game.sRect(px,py,24,24), Game.sPoint(px,py), alphaBlock, FP.zero, false);
				}
			}
			// Create entity
			var s:Stamp = new Stamp(bmp);
			var e:Entity = new Entity(x,y,s);
			s.x = x0*24-x;
			s.y = y0*24-y
			e.layer = 1;
			e.setHitboxTo(s);
			return e;
		}


	}

}

